/*
 *                 Common Public License Notice
 * 
 * The contents of this file are subject to the Common Public License
 * Version 1.0 (the "License"). You may not use this file except in
 * compliance with the License. A copy of the License should have been 
 * provided in the release which contained this file. If none was provided,
 * copies are available at http://www.opensource.org/
 *
 * Copyright (c) 2003 Ascert LLC.
 *
 * Contact: Ascert LLC (info@ascert.com)
 * Contributor(s):
 *      Rob Walker, Ascert LLC
 *
 */
import java.io.IOException;

import com.ascert.comms.x25.X25Factory;
import com.ascert.comms.x25.X25Socket;
import com.ascert.comms.x25.X25SocketFactory;

/**
 *  Test1 - basic socket connection and closedown test.
 *
 * Open's a server socket, attempts to make 20 client connections, pauses until
 * prompted, and then closes down all open sockets.
 *
 * Physical line assignments can also be tested by providing line assignments 
 * for one or both of the server and client sockets e.g.
 *
 *      java ... Test1 0:0 0:1 
 *                      ^   ^
 *                      |   client line assignment
 *                      server line assignment
 *
 *  @version    1.00, 25 April 2003
 *  @author     Rob Walker
 *  @history
 *      25 April 2003    rhw        Created
 */
public class Test1
{
    //////////////////////////////////////////////////
    // STATIC VARIABLES
    //////////////////////////////////////////////////

    public static X25SocketFactory factory;
    
    public static String srvAssign = null;
    public static String cliAssign = null;
    
    //////////////////////////////////////////////////
    // STATIC PUBLIC METHODS
    //////////////////////////////////////////////////

    public static void main (String args[])
            throws Exception
    {
        System.setProperty("com.ascert.comms.driver.farsite.diag",
                "true");
        
        //if enough args supplied, then set server and client socket line
        //assignment information
        if (args.length >= 2)
        {
            srvAssign = args[0];
            cliAssign = args[1];
        }
        
        factory = new X25SocketFactory(null);
        
        // create server socket test thread
        MyServer mySrv = new MyServer();
        Thread srvThread = new Thread(mySrv);
        srvThread.start();
        sleepAwhile(2000);
        
        // create client socket test thread
        MyClient myCli = new MyClient();
        Thread cliThread = new Thread(myCli);
        cliThread.start();
        sleepAwhile(20000);
        
        System.out.print("Press Enter ...");
        System.in.read();
        myCli.cleanup();
        mySrv.cleanup();
    }
    
    //////////////////////////////////////////////////
    // STATIC PROTECTED METHODS
    //////////////////////////////////////////////////
    
    protected static void sleepAwhile(int period)
    {
        try
        {
            Thread.currentThread().sleep(period);
        }
        catch (InterruptedException ie)
        {
        }
    }

    
    protected static void closeAll(X25Socket[] sockArr)
    {
        for (int ix=0; ix < sockArr.length; ix++)
        {
            try
            {   
                if (sockArr[ix] != null)
                {
                    sockArr[ix].shutdown();
                }
                sleepAwhile(200);
            }
            catch (IOException ioe)
            {
                System.out.println("Socket shutdown exception: " + ioe);
            }
        }
    }
    
    //////////////////////////////////////////////////
    // STATIC PRIVATE METHODS
    //////////////////////////////////////////////////

    //////////////////////////////////////////////////
    // INSTANCE VARIABLES
    //////////////////////////////////////////////////

    //////////////////////////////////////////////////
    // CONSTRUCTORS
    //////////////////////////////////////////////////

    //////////////////////////////////////////////////
    // ACCESSOR METHODS
    //////////////////////////////////////////////////

    //////////////////////////////////////////////////
    // PUBLIC INSTANCE METHODS
    //////////////////////////////////////////////////


    //////////////////////////////////////////////////
    // INTERFACE METHODS
    //////////////////////////////////////////////////

    //////////////////////////////////////////////////
    // PROTECTED INSTANCE METHODS
    //////////////////////////////////////////////////

    //////////////////////////////////////////////////
    // PRIVATE INSTANCE METHODS
    //////////////////////////////////////////////////

    //////////////////////////////////////////////////
    // STATIC INNER CLASSES
    //////////////////////////////////////////////////

    public static class MyServer
            implements Runnable
    {
        X25Socket[] socks = new X25Socket[20];
        X25Socket srv;
        
        public void run()
        {
            // basic server bind and accept
            try
            {
                if (srvAssign != null)
                {
                    // force physical adapter/line assignment
                    srv = factory.getSocket();
                    srv.setPhysicalAssignment(srvAssign);
                    srv.bind("23420000222201");
                    srv.listen(3);
                }
                else
                {                    
                    srv = factory.getServerSocket("23420000222201");
                }
                
                for (int ix=0; ix < socks.length; ix++)
                {
                    socks[ix] = srv.accept();
                }
            }
            catch (IOException ioe)
            {
                System.out.println("Server socket exception: " + ioe);
            }
        }

        
        public void cleanup()
        {
            closeAll(socks);
            try
            {   
                if (srv != null)
                {
                    srv.shutdown();
                }
            }
            catch (IOException ioe)
            {
                System.out.println("Server socket exception: " + ioe);
            }
        }
    }

    
    public static class MyClient
            implements Runnable
    {
        X25Socket[] socks = new X25Socket[20];
        
        public void run()
        {
            // basic client connect
            try
            {   
                for (int ix=0; ix < socks.length; ix++)
                {
                    if (cliAssign != null)
                    {
                        // force physical adapter/line assignment
                        socks[ix] = factory.getSocket();
                        socks[ix].setPhysicalAssignment(cliAssign);
                        socks[ix].connect("23420000222201");
                    }
                    else
                    {                    
                        socks[ix] = factory.getClientSocket("23420000222201");
                    }
                    sleepAwhile(250);
                }
            }
            catch (IOException ioe)
            {
                System.out.println("Client socket exception: " + ioe);
            }
        }

        
        public void cleanup()
        {
            closeAll(socks);
        }
    }
                
    //////////////////////////////////////////////////
    // NON-STATIC INNER CLASSES
    //////////////////////////////////////////////////
    
}
