/*
 *  Common Public License Notice
 *
 *  The contents of this file are subject to the Common Public License
 *  Version 1.0 (the "License"). You may not use this file except in
 *  compliance with the License. A copy of the License should have been
 *  provided in the release which contained this file. If none was provided,
 *  copies are available at http://www.opensource.org/
 *
 *  Copyright (c) 2003 Ascert LLC.
 *
 *  Contact: Ascert LLC (info@ascert.com)
 *  Contributor(s):
 *  Rob Walker, Ascert LLC
 *
 */
package com.ascert.comms.driver.farsite;

import java.io.IOException;

import com.ascert.comms.x25.X25Factory;
import com.ascert.comms.x25.X25Socket;

/**
 * Contains the <code>X25Factory</code> implementation for the FarSync range of X.25 cards. Instances of this class can be used to create <code>X25Socket</code> objects that can create and use X.25 connections using the FarSync card.
 *<p>
 * Low level JNI libraries are provided for Windows (<code>farsite.dll</code>) and Linux (<code>libfarsite.so</code>) which perform the require low level API operations to communicate with the FarSync card. The appropriate JNI library will be loaded on initial loading of this class. The specific class loader which loads this class must be able to locate the JNI library using one of the standard mechanisms for loading native libraries (e.g. via the standard Java <code>java.library.path</code> system property).
 *<p>
 * If there is a need to rename the native libraries for any reason, the default library names can be overriden by setting the <code>com.ascert.comms.driver.farsite.libname</code> system property to the name of the new library. This name must be a filename only, and must be in a platform indepdent form (e.g. on Windows, use <code>newfarsite</code> if the library is renamed to <code>newfarsite.dll</code>).
 *
 *
 *
 *@author     Rob Walker
 *@version    1.00, 21 May 2003
 *@since      1.00 
 */
public class X25FactoryImpl
         implements X25Factory
{
    //////////////////////////////////////////////////
    // STATIC VARIABLES
    //////////////////////////////////////////////////

    /** System property to allow override of FarSync native library name */
    public static final String LIBNAME_PROPERTY = "com.ascert.comms.driver.farsite.libname";

    /** Indicates status of low level FarSync JNI loaded. */
    public static boolean libLoaded = false;

    //////////////////////////////////////////////////
    // STATIC PUBLIC METHODS
    //////////////////////////////////////////////////

    //TODO: alternative would be an explicit "initFactory" method?
    static
    {
        try
        {
            String libName = System.getProperty(LIBNAME_PROPERTY);
            if (libName != null)
            {
                System.loadLibrary(libName);
            }
            else
            {
                System.loadLibrary("farsite");
            }
            libLoaded = true;
        }
        catch (Exception ex)
        {
            System.err.println("Exception loading FarSite JNI library: " + ex);
            ex.printStackTrace();
        }
    }


    //////////////////////////////////////////////////
    // STATIC PROTECTED METHODS
    //////////////////////////////////////////////////

    //////////////////////////////////////////////////
    // STATIC PRIVATE METHODS
    //////////////////////////////////////////////////

    //////////////////////////////////////////////////
    // INSTANCE VARIABLES
    //////////////////////////////////////////////////

    //////////////////////////////////////////////////
    // CONSTRUCTORS
    //////////////////////////////////////////////////

    //////////////////////////////////////////////////
    // ACCESSOR METHODS
    //////////////////////////////////////////////////

    //////////////////////////////////////////////////
    // PUBLIC INSTANCE METHODS
    //////////////////////////////////////////////////

    //////////////////////////////////////////////////
    // INTERFACE METHODS - X25Factory
    //////////////////////////////////////////////////

    public X25Socket getSocket()
        throws IOException
    {
        return new X25SocketImpl();
    }

    //////////////////////////////////////////////////
    // PROTECTED INSTANCE METHODS
    //////////////////////////////////////////////////

    //////////////////////////////////////////////////
    // PRIVATE INSTANCE METHODS
    //////////////////////////////////////////////////

    //////////////////////////////////////////////////
    // STATIC INNER CLASSES
    //////////////////////////////////////////////////

    //////////////////////////////////////////////////
    // NON-STATIC INNER CLASSES
    //////////////////////////////////////////////////

}

