/*
 *  Common Public License Notice
 *
 *  The contents of this file are subject to the Common Public License
 *  Version 1.0 (the "License"). You may not use this file except in
 *  compliance with the License. A copy of the License should have been
 *  provided in the release which contained this file. If none was provided,
 *  copies are available at http://www.opensource.org/
 *
 *  Copyright (c) 2003 Ascert LLC.
 *
 *  Contact: Ascert LLC (info@ascert.com)
 *  Contributor(s):
 *  Rob Walker, Ascert LLC
 *
 */
package com.ascert.comms.driver.farsite;

import java.io.IOException;
import java.net.SocketException;
import java.util.StringTokenizer;

import com.ascert.comms.x25.X25Socket;
import com.ascert.comms.x25.X25Message;

/**
 * Contains the <code>X25Socket</code> implementation for the FarSync range of X.25 cards.
 *
 *@author     Rob Walker
 *@version    1.00, 21 May 2003
 *@since      1.00
 */
public class X25SocketImpl
         implements X25Socket
{
    //////////////////////////////////////////////////
    // STATIC VARIABLES
    //////////////////////////////////////////////////

    /** System property controlling whether diagnostics are produced to <code>System.out</code>. Set to <code>true</code> to enable driver diagnostics. */
    public final static String FARSITE_DIAG_PROPERTY =
            "com.ascert.comms.driver.farsite.diag";

    private final static int STATE_INITIAL = 0;
    private final static int STATE_BOUND = 1;
    private final static int STATE_LISTEN = 2;
    private final static int STATE_CONNECTED = 3;
    private final static int STATE_CLOSED = 4;

    private final static byte HDR_QBIT = 0x01;
    private final static byte HDR_MBIT = 0x02;

    private final static int MAX_BUF_LEN = 32767;
    private final static int MAX_FACILITY_LEN = 200;

    private static boolean diag = false;

    //////////////////////////////////////////////////
    // STATIC PUBLIC METHODS
    //////////////////////////////////////////////////

    //////////////////////////////////////////////////
    // STATIC PROTECTED METHODS
    //////////////////////////////////////////////////

    //////////////////////////////////////////////////
    // STATIC PRIVATE METHODS
    //////////////////////////////////////////////////

    static
    {
        String diagStr = System.getProperty(FARSITE_DIAG_PROPERTY);
        if (diagStr != null)
        {
            diag = (diagStr.compareToIgnoreCase("true")) == 0 ? true : false;
        }

        initImpl();
    }


    //////////////////////////////////////////////////
    // INSTANCE VARIABLES
    //////////////////////////////////////////////////

    /** Low level file descriptor for the socket */
    private int sd = -1;

    /** Current state of the socket */
    private int state = STATE_INITIAL;

    /** Lock to force current recv() calls to be sequential */
    private Object rdLock = new Object();

    /** Lock to force current send() calls to be sequential */
    private Object wrLock = new Object();

    /** Indicates whether a physical assignment has been set for the socket */
    private boolean assigned = false;

    /** Indicates current receive buffer size */
    private int rcvBufSize = 0;
    
    //////////////////////////////////////////////////
    // CONSTRUCTORS
    //////////////////////////////////////////////////

    /**
    * General constructor to create and initialise a new X.25 socket. Note that generally this will only be called from within the farsite driver package by the {@link X25FactoryImpl}. 
     *
     *@exception  IOException  Description of the Exception
     */
    protected X25SocketImpl()
        throws IOException
    {
        fskCreate();
    }


    /**
     *  Internally used constructor to create a new X25SocketImpl object with an already known file descriptor i.e. after a file descriptor returned by a low level accept call. 
     *
     *@param  sd allocated file descriptor
     *@exception  IOException  if an error occurs
     */
    private X25SocketImpl(int sd)
        throws IOException
    {
        if (sd == -1)
        {
            throw new IllegalArgumentException("invalid socket descriptor");
        }

        this.sd = sd;
        setState(STATE_CONNECTED);
        // store for later
        this.rcvBufSize = getReceiveBufferSize();
    }

    //////////////////////////////////////////////////
    // ACCESSOR METHODS
    //////////////////////////////////////////////////

    //////////////////////////////////////////////////
    // PUBLIC INSTANCE METHODS
    //////////////////////////////////////////////////

    //////////////////////////////////////////////////
    // INTERFACE METHODS - X25Socket
    //////////////////////////////////////////////////

     public synchronized X25Socket accept()
        throws IOException
    {
        if (this.state != STATE_LISTEN)
        {
            throw new SocketException("can't accept, socket not listening");
        }
        int sdNew = fskAccept();
        //TODO: any close down/tidy up on error?
        return new X25SocketImpl(sdNew);
    }

    /**
     * {@inheritDoc}.
     *<p>
     *<i>FarSite driver specifics:</i><br>
     * For the FarSync card, all NUA values are strings of up to 16 numeric characters.
     */
    public synchronized void bind(String localAddr)
        throws IOException
    {
        validateNUA(localAddr);
        fskBind(localAddr);
        setState(STATE_BOUND);
        //TODO: any close down/tidy up on error?
    }


    public void close()
        throws IOException
    {
        //Note: deliberately not sync'd, to allow close even if blocking recv
        //      is in progress.
        shutdown();
    }

    /**
     * {@inheritDoc}.
     *<p>
     *<i>FarSite driver specifics:</i><br>
     * For the FarSync card, all NUA values are strings of up to 16 numeric characters.
     */
    public synchronized void connect(String remoteAddr)
        throws IOException
    {
        connect(remoteAddr, 0);
    }


    /**
     * {@inheritDoc}.
     *<p>
     *<i>FarSite driver specifics:</i><br>
     * For the FarSync card, all NUA values are strings of up to 16 numeric characters.
     */
    public synchronized void connect(String remoteAddr, int timeout)
        throws IOException
    {
        if (this.state != STATE_INITIAL && this.state != STATE_BOUND)
        {
            throw new SocketException("can't connect, wrong socket state");
        }

        validateNUA(remoteAddr);
        fskConnect(remoteAddr, timeout);
        setState(STATE_CONNECTED);
        // store for later
        this.rcvBufSize = getReceiveBufferSize();
        //TODO: any close down/tidy up on error?
    }


    /** {@inheritDoc}. 
     *<p>
     *<i>FarSite driver specifics:</i><br>
     * At present PVCs are only supported when using the Windows version of the FarSync sockets interface. An <code>IOException</code> with a failure to connect will be seen under Linux.  
     *<p>
     * For the FarSync card, the PVC qualifier should be a numeric string containing the LCN to be used for a PVC, prefixed by a '+' character e.g. "+1" would indicate the PVC should use LCN 1. A phsyical assigment must also have been set for the socket or an exception will be thrown.
     *<p>
     * @see #setPhysicalAssignment
     */
    public synchronized void connectPVC(String pvcQualifier)
        throws IOException
    {
        //for Farsite card a regular connect could also be used, but having
        //separate method forces "PVC" distinction which may be useful for
        //other drivers later. We can also use it to perform validation checks
        if (this.state != STATE_INITIAL)
        {
            throw new SocketException("can't connect PVC, wrong socket state");
        }

        if (!assigned)
        {
            throw new SocketException("PVC must have physical assignment set");
        }

        validateLCN(pvcQualifier);
        fskConnect(pvcQualifier, 0);
        setState(STATE_CONNECTED);
    }


    public boolean isBound()
    {
        return this.state == STATE_BOUND;
    }


    public boolean isConnected()
    {
        return this.state == STATE_CONNECTED;
    }


    public boolean isClosed()
    {
        return this.state == STATE_CLOSED;
    }


    /**
     * {@inheritDoc}.
     *<p>
     *<i>FarSite driver specifics:</i><br>
     * The FarSync card supports a maximum backlog value of 3.
     */
    public synchronized void listen(int backlog)
        throws IOException
    {
        if (this.state != STATE_BOUND)
        {
            throw new SocketException("can't listen, socket not bound");
        }

        //TODO: validate count value (1 to 3)
        fskListen(backlog);
        setState(STATE_LISTEN);
    }


    public int recv(X25Message msg)
        throws IOException
    {
        if (this.state != STATE_CONNECTED)
        {
            throw new SocketException("can't recv, socket not connected");
        }

        int len = msg.getLen();
        if (len <= 0 || len < this.rcvBufSize)
        {
            throw new IllegalArgumentException("receiving buffer too small.");
        }
        
        int lenIn = -1;
        synchronized (this.rdLock)
        {
            //TODO: could use per-socket buffer to avoid obj. creation cost
            byte[] buf = new byte[len + 1];
            lenIn = fskRecv(buf, len + 1);

            if (lenIn > 0)
            {
                System.arraycopy(buf, 1, msg.getMsg(), 0, lenIn - 1);
                msg.setQbit((buf[0] & HDR_QBIT) != 0);
                msg.setMbit((buf[0] & HDR_MBIT) != 0);
                msg.setLen(lenIn - 1);
            }
            else
            {
                msg.setLen(lenIn);
            }
        }
        return lenIn;
    }


    public int send(X25Message msg)
        throws IOException
    {
        if (this.state != STATE_CONNECTED)
        {
            throw new SocketException("can't send, socket not connected");
        }

        //TODO: farsite has max 32k send, need to validate/handle this
        int lenOut = -1;
        synchronized (this.wrLock)
        {
            //TODO: could use per-socket buffer to avoid obj. creation cost
            // create X.25 buffer with Q/M bit header
            int len = msg.getLen();
            byte[] buf = new byte[len + 1];
            System.arraycopy(msg.getMsg(), 0, buf, 1, len);
            buf[0] = msg.getQbit() ? HDR_QBIT : 0;
            buf[0] |= msg.getMbit() ? HDR_MBIT : 0;

            lenOut = fskSend(buf, len + 1);
        }
        return lenOut;
    }


    public void shutdown()
        throws IOException
    {
        // mark closed at start of shutdown, to prevent further use
        int currState = state;
        setState(STATE_CLOSED);        
        //Note: deliberately not sync'd, to allow close even if blocking recv
        //      is in progress.
        switch (currState)
        {
            case STATE_INITIAL:
            case STATE_BOUND:
                fskClose();
                break;
            
            case STATE_CLOSED:
                break;
            
            case STATE_CONNECTED:
                fskShutdown();
                fskClose();
                break;
            
            case STATE_LISTEN:
                fskClose();
                break;
        }
    }


    public String getLocalAddress()
    {
        if (this.state != STATE_BOUND &&
                this.state != STATE_LISTEN &&
                this.state != STATE_CONNECTED)
        {
            return null;
        }
        return fskGetAddr(true);
    }


    public String getRemoteAddress()
    {
        if (this.state != STATE_CONNECTED)
        {
            return null;
        }
        return fskGetAddr(false);
    }


    public byte[] getCallUserData()
        throws IOException
    {
        return fskGetCUD(false);
    }


    public void setCallUserData(byte[] cud)
        throws IOException
    {
        validateCUD(cud);
        fskSetCUD(cud, false);
    }


    public byte[] getClearUserData()
        throws IOException
    {
        return fskGetCUD(true);
    }


    public void setClearUserData(byte[] cud)
        throws IOException
    {
        validateCUD(cud);
        fskSetCUD(cud, true);
    }


    public byte[] getClearCodes()
        throws IOException
    {
        return fskGetClearCodes();
    }


    public void setClearCodes(byte[] clr)
        throws IOException
    {
        if (clr == null)
        {
            throw new IllegalArgumentException("supplied diags are null");
        }

        if (clr.length != 2)
        {
            throw new IllegalArgumentException("Diags exceed maximum length");
        }
        fskSetClearCodes(clr);
    }

    /**
     *@see #setPhysicalAssignment
     */
    public String getPhysicalAssignment()
        throws IOException
    {
        if (this.state == STATE_CLOSED)
        {
            throw new SocketException("can't get adapter, socket closed");
        }

        int[] adapt = fskGetAdapter();
        return new String(adapt[0] + ":" + adapt[1]);
    }

    /**
     * {@inheritDoc}.
     *<p>
     *<i>FarSite driver specifics:</i><br>
     * For the FarSync card, the physical assignment is a string in the format 'adaptor:line' e.g. an assignment value of "0:1" would tie the socket to adapter 0, line 1.
     */
    public void setPhysicalAssignment(String param)
        throws IOException
    {
        StringTokenizer st = new StringTokenizer(param, ":");
        if (st == null || st.countTokens() < 2)
        {
            throw new IllegalArgumentException("bad assignment string");
        }

        int[] adapt = new int[2];
        try
        {
            adapt[0] = Integer.parseInt(st.nextToken());
            adapt[1] = Integer.parseInt(st.nextToken());
        }
        catch (NumberFormatException nfe)
        {
            throw new IllegalArgumentException("bad adapter or line number");
        }

        assigned = true;
        fskSetAdapter(adapt);
    }

    public byte[] getRawFacilities()
        throws IOException
    {
        return fskGetRawFacilities();
    }


    public void setRawFacilities(byte[] fac)
        throws IOException
    {
        if (fac == null)
        {
            throw new IllegalArgumentException("supplied facilities are null");
        }

        if (fac.length > MAX_FACILITY_LEN)
        {
            throw new IllegalArgumentException(
                    "Facilities exceed maximum length");
        }
        fskSetRawFacilities(fac);
    }


    public int getReceiveBufferSize()
        throws IOException
    {
        return fskGetRcvBuf();
    }


    public void setReceiveBufferSize(int size)
        throws IOException
    {
        if (size <= 0 || size > MAX_BUF_LEN)
        {
            throw new IllegalArgumentException("invalid buffer size");
        }
        fskSetRcvBuf(size);
        this.rcvBufSize = size;
    }

    
    //////////////////////////////////////////////////
    // PROTECTED INSTANCE METHODS
    //////////////////////////////////////////////////

    //////////////////////////////////////////////////
    // PRIVATE INSTANCE METHODS
    //////////////////////////////////////////////////

    private void setState(int newState)
    {
        if (diag)
        {
            System.out.println("X25 socket: " + this.toString() + ", previous state=" + this.state + ", new state=" + newState);
        }
        this.state = newState;
    }


    private void validateNUA(String nua)
        throws IOException
    {
        if (nua == null)
        {
            throw new IllegalArgumentException("supplied NUA is null");
        }

        if (nua.length() > X25_NUA_ADDRLEN)
        {
            throw new IllegalArgumentException("NUA exceeds maximum length");
        }
    }


    private void validateLCN(String lcn)
        throws IOException
    {
        if (lcn == null)
        {
            throw new IllegalArgumentException("supplied LCN is null");
        }

        if (!lcn.startsWith("+"))
        {
            throw new IllegalArgumentException(
                    "LCN specifier must start with '+'");
        }

        try
        {
            int num = Integer.parseInt(lcn.substring(1));
            if (num < 0 || num > 0xfff)
            {
                throw new IllegalArgumentException("LCN not in valid range");
            }
        }
        catch (NumberFormatException nfe)
        {
            throw new IllegalArgumentException("LCN not a valid number");
        }
    }


    private void validateCUD(byte[] cud)
        throws IOException
    {
        if (cud == null)
        {
            throw new IllegalArgumentException("supplied CUD is null");
        }

        if (cud.length > X25_CUD_LENGTH)
        {
            throw new IllegalArgumentException("CUD exceeds maximum length");
        }
    }

    //////////////////////////////////////////////////
    // NATIVE METHODS
    //////////////////////////////////////////////////

    private native static void initImpl();
    private native void fskCreate()
        throws IOException;
    private native void fskBind(String localAddr)
        throws IOException;
    private native void fskListen(int count)
        throws IOException;
    private native int fskAccept()
        throws IOException;
    private native void fskConnect(String remoteAddr, int timeout)
        throws IOException;
    private native void fskClose()
        throws IOException;
    private native void fskShutdown()
        throws IOException;
    private native int fskSend(byte[] buf, int len)
        throws IOException;
    private native int fskRecv(byte[] buf, int len)
        throws IOException;
    private native byte[] fskGetCUD(boolean clr)
        throws IOException;
    private native void fskSetCUD(byte[] cud, boolean clr)
        throws IOException;
    private native byte[] fskGetClearCodes()
        throws IOException;
    private native void fskSetClearCodes(byte[] clr)
        throws IOException;
    private native byte[] fskGetRawFacilities()
        throws IOException;
    private native void fskSetRawFacilities(byte[] fac)
        throws IOException;
    private native String fskGetAddr(boolean local);
    private native int[] fskGetAdapter();
    private native void fskSetAdapter(int[] adapt);
    private native int fskGetRcvBuf()
        throws IOException;
    private native void fskSetRcvBuf(int size)
        throws IOException;

    //////////////////////////////////////////////////
    // STATIC INNER CLASSES
    //////////////////////////////////////////////////

    //////////////////////////////////////////////////
    // NON-STATIC INNER CLASSES
    //////////////////////////////////////////////////

}

