/*
 *  Common Public License Notice
 *
 *  The contents of this file are subject to the Common Public License
 *  Version 1.0 (the "License"). You may not use this file except in
 *  compliance with the License. A copy of the License should have been
 *  provided in the release which contained this file. If none was provided,
 *  copies are available at http://www.opensource.org/
 *
 *  Copyright (c) 2003 Ascert LLC.
 *
 *  Contact: Ascert LLC (info@ascert.com)
 *  Contributor(s):
 *  Rob Walker, Ascert LLC
 *
 */
package com.ascert.comms.x25;

/**
 *  Provides a common class encapsulating X.25 messages to be sent or received. In addition to containing a data buffer with contents of the message, the class also allows setting or retrieval of message status indicators such as the X.25 M-Bit ot Q-Bit.
 *
 *@author     Rob Walker
 *@version    1.00, 21 May 2003
 *@since 1.00
 */
public class X25Message
{
    //////////////////////////////////////////////////
    // STATIC VARIABLES
    //////////////////////////////////////////////////

    //////////////////////////////////////////////////
    // STATIC PUBLIC METHODS
    //////////////////////////////////////////////////

    //////////////////////////////////////////////////
    // STATIC PROTECTED METHODS
    //////////////////////////////////////////////////

    //////////////////////////////////////////////////
    // STATIC PRIVATE METHODS
    //////////////////////////////////////////////////

    //////////////////////////////////////////////////
    // INSTANCE VARIABLES
    //////////////////////////////////////////////////

    /**  Indicates status of the X.25 Q-bit when this message is sent or received. */
    protected boolean qBit = false;

    /**  Indicates status of the X.25 M-bit when this message is sent or received. The M-bit shows whether more data needs to be sent or received to to complete the message. */
    protected boolean mBit = false;

    /**  Buffer containing the message data to be sent or received. */
    protected byte[] msg;

    /**  Indicates the length of data to be sent or which was received. */
    protected int len;

    //////////////////////////////////////////////////
    // CONSTRUCTORS
    //////////////////////////////////////////////////

    /**
     *  Constructs an X25Message object, using the supplied buffer for sending and receiving data. The length to send or receive will be set to the length of the supplied buffer. Both the Q-bit and M-bit will be set <code>false</code>
     *
     *@param  msg  buffer containg data to be sent, or in which received data will be placed
     */
    public X25Message(byte[] msg)
    {
        this(msg, msg.length, false, false);
    }


    /**
     *  Constructs an X25Message object, using the supplied buffer for sending and receiving data. The length to send or receive will be set to the length of the supplied buffer. The Q-bit and M-bit will be set to the values supplied.
     *
     *@param  msg  buffer containg data to be sent, or in which received data will be placed
     *@param  qBit  status of the Q-bit in the constructed message
     *@param  mBit  status of the M-bit in the constructed message
     */
    public X25Message(byte[] msg, boolean qBit, boolean mBit)
    {
        this(msg, msg.length, qBit, mBit);
    }


    /**
     *  Constructs an X25Message object, using the supplied buffer for sending and receiving data. The length to send or receive, and the Q-bit and M-bit will be set to the values supplied.
     *
     *@param  msg  buffer containg data to be sent, or in which received data will be placed
     *@param  len   length of data to be sent, or maximum length of data to be received
     *@param  qBit  status of the Q-bit in the constructed message
     *@param  mBit  status of the M-bit in the constructed message
     */
    public X25Message(byte[] msg, int len, boolean qBit, boolean mBit)
    {
        this.msg = msg;
        this.len = len;
        this.qBit = qBit;
        this.mBit = mBit;
    }


    //////////////////////////////////////////////////
    // ACCESSOR METHODS
    //////////////////////////////////////////////////

    /**
     *  Gets the Q-bit attribute of the X25Message object
     *
     *@return    The Q-bit value
     *@see #setQbit
     */
    public boolean getQbit()
    {
        return this.qBit;
    }


    /**
     *  Sets the Q-bit attribute of the X25Message object
     *
     *@param  qBit  The new Q-bit value
     *@see #getQbit
     */
    public void setQbit(boolean qBit)
    {
        this.qBit = qBit;
    }


    /**
     *  Gets the M-bit attribute of the X25Message object
     *
     *@return    The M-bit value
     *@see #setMbit
     */
    public boolean getMbit()
    {
        return this.mBit;
    }


    /**
     *  Sets the M-bit attribute of the X25Message object
     *
     *@param  mBit  The new M-bit value
     *@see #getMbit
     */
    public void setMbit(boolean mBit)
    {
        this.mBit = mBit;
    }


    /**
     *  Gets the msg attribute of the X25Message object
     *
     *@return    reference to the msg buffer supplied when the message was created
     */
    public byte[] getMsg()
    {
        return this.msg;
    }


    /**
    *  Gets the length attribute of the X25Message object. On initiating a {@link X25Socket#send} operation this value should contain the length of the data to be sent from the {@link #msg} buffer. On initiation of a {@link X25Socket#recv} operation this value should contain the maximum length of data to be received. On completion of the <code>X25Socket.recv</code> operation the value will contain the actual number of bytes received.
     *
     *@return    the current length value
     *@see #setLen
     */
    public int getLen()
    {
        return this.len;
    }


    /**
     *  Sets the length attribute of the X25Message object
     *
     *@param  len  The new len value
     *@see #getLen
     */
    public void setLen(int len)
    {
        this.len = len;
    }

    //////////////////////////////////////////////////
    // PUBLIC INSTANCE METHODS
    //////////////////////////////////////////////////

    //////////////////////////////////////////////////
    // INTERFACE METHODS
    //////////////////////////////////////////////////

    //////////////////////////////////////////////////
    // PROTECTED INSTANCE METHODS
    //////////////////////////////////////////////////

    //////////////////////////////////////////////////
    // PRIVATE INSTANCE METHODS
    //////////////////////////////////////////////////

    //////////////////////////////////////////////////
    // STATIC INNER CLASSES
    //////////////////////////////////////////////////

    //////////////////////////////////////////////////
    // NON-STATIC INNER CLASSES
    //////////////////////////////////////////////////

}

