/*
 *  Common Public License Notice
 *
 *  The contents of this file are subject to the Common Public License
 *  Version 1.0 (the "License"). You may not use this file except in
 *  compliance with the License. A copy of the License should have been
 *  provided in the release which contained this file. If none was provided,
 *  copies are available at http://www.opensource.org/
 *
 *  Copyright (c) 2003 Ascert LLC.
 *
 *  Contact: Ascert LLC (info@ascert.com)
 *  Contributor(s):
 *  Rob Walker, Ascert LLC
 *
 */
package com.ascert.comms.x25;

import java.io.IOException;

import java.net.SocketException;
import java.net.ConnectException;
import java.net.BindException;
import java.net.NoRouteToHostException;

/** 
 * Defines a common interface for connecting and communicating using X.25. For convenience a socket based approach has been chosen for the naming and usage of methods. This will be familiar to most Java programmers from the <code>java.net.*</code> TCP/IP classes. Driver implementations for specific vendor's cards and software API's must map this interface onto card and driver specific calls to perform the required functions.
 * 
 * @author Rob Walker
 * @version 1.00, 21 May 2003
 * @since 1.00
 */
public interface X25Socket
{
    // TODO: control of send/recv buffer size??
    
    // TODO: obtain details of the maxium message size??

    
    //////////////////////////////////////////////////
    // STATIC VARIABLES
    //////////////////////////////////////////////////

	/** Constant value for maximum size of Call User Data. Note that for non fast-select calls this is reduced to only 16 bytes.	 */
    public final static int X25_CUD_LENGTH = 128;
	/** Constant value for the maximum size of an X.25 NUA.	 */
    public final static int X25_NUA_ADDRLEN = 16;


    //////////////////////////////////////////////////
    // SIGNATURES
    //////////////////////////////////////////////////

	/** 
	 * Accept a new incoming X.25 SVC (switched virtual circuit) connection on a server socket. Prior to accepting calls, server sockets must be bound to an NUA and be listening for incoming connections. A new socket instance will be returned for the incoming call, which cannot be <code>null</code>. Error cases will be signalled by an appropriate exception being thrown. This method may be called many times for a single server socket e.g. to accept multiple incoming calls for the same NUA.
	 * 
	 * @since 1.00
	 * @exception IOException if an error occurs whilst waiting for a call
	 * @return a new <code>X25Socket</code> instance for the incoming call
	 * @see #bind
	 * @see #listen
	 */
    public X25Socket accept()
        throws IOException;


	/** 
	 * Binds a socket to a specific local NUA. This method is typically used to bind server sockets to local NUA's on which they will receive incoming calls. It can also be used to force client sockets to make calls from a specific local NUA. 
	 * 
	 * @since 1.00
	 * @exception IOException if an error occurs during the bind
	 * @param localAddr local NUA to bind the socket too.
	 */
    public void bind(String localAddr)
        throws IOException;


	/** 
	 * Closes a socket, disconnecting any call. Upon completion no further operations may be performed on the socket and no further data can be sent or received.
	 * 
	 * @since 1.00
	 * @exception IOException if an error occurs during the close
	 */
    public void close()
        throws IOException;


	/** 
	 * Connects a client socket to a specified remote NUA using an X.25 SVC. The client socket can be bound to a local NUA. If the socket is already be connected or has been closed an Exception will be thrown.
	 * 
	 * @since 1.00
	 * @exception IOException if an error occurs during the connect
	 * @param remoteAddr remote NUA to connect the socket too
     * @see #connect(String, int)
	 */
    public void connect(String remoteAddr)
        throws IOException;


	/** 
    * Connects a client socket with an optional timeout value, which may be 0 if no timeout is required. Note that there is no guarantee that all driver implementations will support timeouts, in which case this call and {@link #connect(String)} are synonymous.
	 * 
	 * @since 1.00
	 * @exception IOException if an error occurs during the connect
	 * @param remoteAddr remote NUA to connect the socket too
	 * @param timeout timeout value in milliseconds, or 0 if no timeout is required
	 */
    public void connect(String remoteAddr, int timeout)
        throws IOException;


	/** 
	 * Connects a client socket to a PVC (permanent virtual circuit) X.25 connection. Note that although the "connect" metaphor is used here, in fact PVC's are permanent. So the only connection which is actually being made is between the Java application and the local X.25 card's i/o channel for the specific PVC. 
     *<p>
     *Note that whilst the format of NUA's for SVC calls follow a standard which is likely to be common to most X.25 driver implementations, the means of specifying a particular PVC to be used may vary across implementations. Typically the format will be some qualifier to the specific LCN (logical channel number) to be used for the connection.  
	 * 
	 * @since 1.00
	 * @exception IOException if an error occurs connecting to the PVC
	 * @param pvcQualifier qualifier for the PVC (e.g. LCN number)
	 */
    public void connectPVC(String pvcQualifier)
        throws IOException;


	/** 
	 * Tests whether the X25Socket is bound
	 * 
	 * @since 1.00
	 * @return true if bound, false otherwise
     * @see #bind
	 */
    public boolean isBound();


	/** 
	 * Tests whether the X25Socket is connected, and hence available for data transfer.
	 * 
	 * @since 1.00
	 * @return true if connected, false otherwise 
     * @see #connect
	 */
    public boolean isConnected();


	/** 
	 * Tests whether the X25Socket is closed, and hence unavailable for any further operations.
	 * 
	 * @since 1.00
	 * @return true if closed, false otherwise
     * @see #close
	 */
    public boolean isClosed();


	/** 
	 * Puts a server socket into Listen Mode, ready to accept incoming connections. Note that prior to listening for connections server sockets must have been bound to a local NUA.
	 * 
	 * @since 1.00
	 * @exception IOException if an error occurs during the listen
	 * @param backlog number of incoming connections that can be queued pending an accept operation (may not be supported by all driver implementations)
     * @see #bind
     * @see #accept
	 */
    public void listen(int backlog)
        throws IOException;


	/** 
	 * Receive an incoming message on an X.25 socket. An exception will be thrown if the socket is not connected, or if any other error occurs whilst waiting for data.
     *<p>
     * It should be note that X25 connections are packet based, hence the operation of this call is subtly different to stream based protocols such as TCP/IP. Firstly, if the buffer within the <code>X25Message</code> is large enough to receive a complete message, then it can be relied upon that whole messages will be received which are neither segmented nor chained with data from other messages. If the incoming message is too large to fit in the supplied <code>X25Message</code>, then as much data as will fit in the buffer will be returned, and the M-Bit will be set to indicate there is more data to be received to complete the incoming message. Further <code>recv</code> calls should be posted until a final <code>X25Message</code> is received with the M-bit unset.
     *<p>
     * Additionally, the presence of a Q-bit message can also be determined from the returned X25Message object. 
	 * 
	 * @since 1.00
	 * @exception IllegalArgumentException if the supplied buffer size is smaller than the current receive buffer size
	 * @exception IOException if an error occurs whilst receiving data
	 * @param msg message object in which to receive incoming data
	 * @return count of the number of bytes received, or 0 if the socket becomes disconnected whilst awaiting data
     * @see #getReceiveBufferSize
     * @see #setReceiveBufferSize
	 */
    public int recv(X25Message msg)
        throws IOException;


	/** 
	 * Send a message on an X.25 socket. An exception will be thrown if the socket is not connected, or if any other error occurs whilst sending the data.
     *<p>
     * The <code>X25Message</code> object can specify whether the data is sent with one or both of the Q-bit and the M-bit set. Message sent with the M-bit set may be buffered by the driver implementation or the network until either full packets of data are available to send, or until a final non M-bit <code>X25Message</code> has been sent. 
	 * 
	 * @since 1.00
	 * @exception IOException if an error occurs whilst sending the data
	 * @param msg message object containing the data to be sent
	 * @return count of the number of bytes actually sent
     * @see #recv
	 */
    public int send(X25Message msg)
        throws IOException;


	/** 
    * Graceful shutdown of the socket. Since graceful shutdown is not really a standard X.25 feature, some driver implementations may take this method and {@link #close} as being synonymous. Where possible though, shutdown should ensure that sockets are closed in a manner which cleans up outstanding i/o requests in low level drivers.
	 * 
	 * @since 1.00
	 * @exception IOException if an error occurs during the shutdown
     * @see #close
	 */
    public void shutdown()
        throws IOException;


	/** 
	 * Gets the local NUA to which the socket is bound (if any).
	 * 
	 * @since 1.00
	 * @return local NUA the socket is bound to
	 */
    public String getLocalAddress();


	/** 
	 * Gets the remote NUA to which the socket is connected (if any).
	 * 
	 * @since 1.00
	 * @return local NUA the socket is connected to
	 */
    public String getRemoteAddress();


	/** 
	 * Gets any User Data which was supplied with an incoming call. This method will generally only return valid data immediately after a socket has been connected. 
	 * 
	 * @since 1.00
	 * @exception IOException if an error occurs getting the call user data
	 * @return any user data supplied with an incoming call, or a 0 length array if none was supplied
     * @see #accept
	 */
    public byte[] getCallUserData()
        throws IOException;


	/** 
	 * Sets the User Data to be included in the next outgoing call. This method is generally only valid immediately before making a new connection. Up to 128 bytes of user data (fast select calls) in an X.25 call, or 16 bytes of user data (normal calls).
	 * 
	 * @since 1.00
	 * @exception IOException if an error occurs setting the call user data
	 * @param cud the user data to be sent with the next call
     * @see #connect
	 */
    public void setCallUserData(byte[] cud)
        throws IOException;


	/** 
	 * Gets any User Data which was supplied when the current call was cleared. This method will generally only return valid data immediately after a socket has been disconnected.
	 * 
	 * @since 1.00
	 * @exception IOException if an error occurs getting the clear user data
	 * @return any user data supplied when the call was disconnected, or a 0 length array if none was supplied
	 */
    public byte[] getClearUserData()
        throws IOException;


	/** 
	 * Sets the User Data to be included when the current call is cleared. This method is generally only valid immediately before disconnecting a call. 
	 * 
	 * @since 1.00
	 * @exception IOException if an error occurs setting the clear user data
	 * @param cud the user data to be sent when the call is cleared
     * @see #close
	 */
    public void setClearUserData(byte[] cud)
        throws IOException;


	/** 
	 * Gets any cause and diagnostic codes supplied when the current call was disconnected. Typically this method is only applicable once a call has been disconnected.
	 * 
	 * @since 1.00
	 * @exception IOException if an error occurs getting the clear codes
	 * @return 2 byte array, byte[0] being the cause and byte[1] being the diagnostic data
	 */
    public byte[] getClearCodes()
        throws IOException;


	/** 
	 * Sets the cause and diagnostic codes to be supplied when the current call is disconnected. Typically this method is only applicable prior to disconnecting a call. Note that setting cause & diagnostic codes may affect the error condition seen by the remote party on disconnection, and if the remote party is also using this Java library it may also affect which (if any) <code>IOException</code> is thrown. 
	 * 
	 * @since 1.00
	 * @exception IOException if an error occurs setting the clear codes
	 * @param clr 2 byte array, byte[0] being the cause and byte[1] being the diagnostic data
     * @see #close
	 */
    public void setClearCodes(byte[] clr)
        throws IOException;


    //note: format of param is vendor specific
    //      (could expand to include driver vendor naming)
	/** 
	 * Gets any physical assignment properties for the socket.
	 * 
	 * @since 1.00
	 * @exception IOException if an error occurs getting the physical assignment details
	 * @return any physical assignment details which are applicable
	 */
    public String getPhysicalAssignment()
        throws IOException;
        
	/** 
	 * Sets the physical assignment properties for the socket. 
     * <p>
     * It is common for servers to have multi-port X.25 cards, each port driving a physically separate X.25 lines. Additionally, some servers may also have multiple X.25 cards installed. Depending on network topology, there may be a need to ensure that certain operations such as {@link #bind}, {@link #accept}, {@link #connect} are directed to only a single line e.g. where an NUA is only valid for a single physical link. This method allows a socket to be tied to a specific physical line. Note, however, that at present the format used for the physical assignment specifier is driver implementation specific.  
	 * 
	 * @since 1.00
	 * @exception IOException if an error occurs setting the physical assignment
	 * @param param driver implementation specific details of the physical line to which all operations for this socket should be directed
	 */
    public void setPhysicalAssignment(String param)
        throws IOException;


	/** 
	 * Gets any X.25 Facilities supplied with an incoming call. Typically only of use immediately after a new connection has been received. The returned facilities will be in the raw format used in the X.25 incoming call packet.
	 * 
	 * @since 1.00
	 * @exception IOException if an error occurs getting the raw facilities
	 * @return any X.25 Facilities received in the incoming call
     * @see #accept
	 */
    public byte[] getRawFacilities()
        throws IOException;


	/** 
	 * Sets the X.25 Facilities to be used with the next outgoing call. Typically only of use immediately before making a new connection. The facilities supplied must be in the raw format used for the X.25 outgoing call packet.
	 * 
	 * @since 1.00
	 * @exception IOException if an error occurs setting the raw facilities
	 * @param fac raw facilities to be included in outgoing call
     * @see #connect
	 */
    public void setRawFacilities(byte[] fac)
        throws IOException;
        
	/** 
	 * Gets the buffer size used by the underlying driver implementation for input on this connection.
	 * 
	 * @since 1.00
	 * @exception IOException if an error occurs getting the size
	 * @return size of input buffer used for receiving data on this <code>X25Socket</code>
     * @see #setReceiveBufferSize
	 */
    public int getReceiveBufferSize()
        throws IOException;

	/** 
	 * Sets the buffer size used by the underlying driver implementation for input on this connection.
     *<p>
     * The receive buffer size can affect the ability of the X.25 driver implementation to assemble incoming packets with the M-bit set into whole messages. Setting this value too small will result in lot's of message fragments being received by the application, and more message re-assembly needing to be handled in the application code.
     *<p>
     * Whatever value is used for the receive buffer size, applications must always ensure they supply a buffer of at least this size in {@link #recv} calls to avoid an <code>IllegalArgumentException</code> being thrown.  
	 * 
	 * @since 1.00
	 * @exception IllegalArgumentException if the size is less than 0 or greater than the maximum allowed by the driver implementation.
	 * @exception IOException if an error occurs setting the raw facilities
	 * @param size the new size for the input buffer. The value must be greater than 0.
     * @see #getReceiveBufferSize
     * @see #recv
	 */
    public void setReceiveBufferSize(int size)
        throws IOException;
        
}

