/*
 *  Common Public License Notice
 *
 *  The contents of this file are subject to the Common Public License
 *  Version 1.0 (the "License"). You may not use this file except in
 *  compliance with the License. A copy of the License should have been
 *  provided in the release which contained this file. If none was provided,
 *  copies are available at http://www.opensource.org/
 *
 *  Copyright (c) 2003 Ascert LLC.
 *
 *  Contact: Ascert LLC (info@ascert.com)
 *  Contributor(s):
 *  Rob Walker, Ascert LLC
 *
 */
package com.ascert.comms.x25;

import java.io.IOException;

/**
*  Convenience class for creating X.25 sockets using driver specific {@link X25Factory} implementations.
 *
 *@author     Rob Walker
 *@version    1.00, 21 May 2003
 *@since      1.00
 */
public class X25SocketFactory
{
    //////////////////////////////////////////////////
    // STATIC VARIABLES
    //////////////////////////////////////////////////

    /** default factory to use for constructing X.25 sockets */
    private static X25Factory dfltImpl;

    //////////////////////////////////////////////////
    // STATIC PUBLIC METHODS
    //////////////////////////////////////////////////

    static
    {
        // See if we have a system property for the X.25 provider, and if so
        // use it to set our factory
        try
        {
            String factory = System.getProperty(
                    X25Factory.FACTORY_CLASS_PROPERTY);
            if (factory != null)
            {
                Class clazz = Class.forName(factory);
                dfltImpl = (X25Factory) clazz.newInstance();
            }
        }
        catch (ClassNotFoundException ce)
        {
            //TODO: log error??
            // leave as null
        }
        catch (InstantiationException ie)
        {
            //TODO: log error??
            // leave as null
        }
        catch (Exception e)
        {
            //TODO: log error??
            // leave as null
        }
    }


    /**
    *  Sets the default <code>X25Factory</code> to be used for constructing X.25 socket objects where no specific driver factory is used. On initial loading of the {@link X25SocketFactory} class an attempt will be made to set the default value using the fully qualified class name specific in the {@link X25Factory#FACTORY_CLASS_PROPERTY} System property.
     *
     *@param  impl  The new default X25Factory value
     */
    public static void setDefaultX25Factory(X25Factory impl)
    {
        dfltImpl = impl;
    }


    //////////////////////////////////////////////////
    // STATIC PROTECTED METHODS
    //////////////////////////////////////////////////

    //////////////////////////////////////////////////
    // STATIC PRIVATE METHODS
    //////////////////////////////////////////////////

    //////////////////////////////////////////////////
    // INSTANCE VARIABLES
    //////////////////////////////////////////////////

    /**  Driver specific factory implementation to be used for creating X.25 sockets */
    protected X25Factory impl;

    //////////////////////////////////////////////////
    // CONSTRUCTORS
    //////////////////////////////////////////////////

    /**
     *  Constructor a new <code>X25SocketFactory</code> using either the supplied <code>X25Factory</code> implementation, or the default value if the supplied value is <code>null</code>.
     *
     *@param  impl  specific <code>X25Factory</code> implementation to use for socket creation, or null if the default is to be used
     */
    public X25SocketFactory(X25Factory impl)
    {
        this.impl = (impl != null) ? impl : dfltImpl;
    }

    //////////////////////////////////////////////////
    // ACCESSOR METHODS
    //////////////////////////////////////////////////

    //////////////////////////////////////////////////
    // PUBLIC INSTANCE METHODS
    //////////////////////////////////////////////////

    /**
     *  Creates a new X.25 socket using the factory implementation supplied at construction time.
     *
     *@return     newly created X.25 socket
     *@exception  IOException  if an error occurs constructing the socket
     */
    public X25Socket getSocket()
        throws IOException
    {
        if (this.impl == null)
        {
            throw new IOException("no driver factory class available");
        }

        return this.impl.getSocket();
    }


    /**
    *  Creates a new X.25 server socket bound to a specific local NUA, and ready to {@link X25Socket#accept} incoming connections.
     *
     *@param  localAddr local address to listen for incoming connections
     *@return  newly create server socket
     *@exception  IOException  if an error occurs creating or initialising the server socket
     */
    public X25Socket getServerSocket(String localAddr)
        throws IOException
    {
        X25Socket sock = this.getSocket();
        sock.bind(localAddr);
        sock.listen(3);
        return sock;
    }


    /**
    *  Creates a new X.25 client socket connected to a specific remote NUA. On successful return, the socket will already be connected to the NUA and be available for data transfer.
     *
     *@param  remoteAddr remote NUA to connect to
     *@return  created socket connected to the remote NUA
     *@exception  IOException if an error occurs creating the socket or connecting to the NUA
     */
    public X25Socket getClientSocket(String remoteAddr)
        throws IOException
    {
        X25Socket sock = this.getSocket();
        sock.connect(remoteAddr);
        return sock;
    }

    //////////////////////////////////////////////////
    // INTERFACE METHODS
    //////////////////////////////////////////////////

    //////////////////////////////////////////////////
    // PROTECTED INSTANCE METHODS
    //////////////////////////////////////////////////

    //////////////////////////////////////////////////
    // PRIVATE INSTANCE METHODS
    //////////////////////////////////////////////////

    //////////////////////////////////////////////////
    // STATIC INNER CLASSES
    //////////////////////////////////////////////////

    //////////////////////////////////////////////////
    // NON-STATIC INNER CLASSES
    //////////////////////////////////////////////////

}

