/*
 *                 Common Public License Notice
 * 
 * The contents of this file are subject to the Common Public License
 * Version 1.0 (the "License"). You may not use this file except in
 * compliance with the License. A copy of the License should have been 
 * provided in the release which contained this file. If none was provided,
 * copies are available at http://www.opensource.org/
 *
 * Copyright (c) 2003 Ascert LLC.
 *
 * Contact: Ascert LLC (info@ascert.com)
 * Contributor(s):
 *      Rob Walker, Ascert LLC
 *
 */

#include <stdio.h>
#include <stdlib.h>
#include <sys/types.h>
#include <errno.h>
#include <string.h>
#include <sys/ioctl.h>
#include <sys/socket.h>
#include <linux/socket.h>
#include <linux/sockios.h>
#include <linux/x25.h>
#include <fsx25sockapi.h>

#include "jni.h"

#include "x25utl.h"
#include "platform.h"

/*
 * Table used to map known error conditions, to appropriate error messages 
 * and exception types. The following are the main specific errors that the 
 * farsite api will return (from Linux sockets API documentation):
 *
 *  Socket creation:
 *   EPROTONOSUPPORT, ENFILE, EMFILE, EACCES, ENOBUFS, ENOMEM, EINVAL, ENODEV
 * 
 *  Bind-type operations (includes connecting on unbound sockets):
 *   EBADF, EINVAL, ENOTSOCK, EACCES, ENOSPC
 *  
 *  Listen-type operations:
 *   EADDRINUSE, EBADF, ENOTSOCK, EOPNOTSUPP, EINVAL, ENOSPC
 *
 *  Accept-type operations:
 *   EADDRINUSE, EBADF, ENOTSOCK, EOPNOTSUPP, EINVAL, EWOULDBLOCK, ENOSPC,
 *   EINTR
 *  
 *  Connect-type operations:
 *   EBADF, EFAULT, ENOTSOCK, EISCONN, ECONNREFUSED, ETIMEDOUT, ENETUNREACH,
 *   EADDRINUSE, EINPROGRESS, EALREADY, EAGAIN, EAFNOSUPPORT, ENOSPC
 * 
 *  Close-type operations: (including shutdown)
 *   EBADF, EINTR, EIO, ENOTSOCK, ENOTCONN
 *
 *  Data transfer operations:
 *   EBADF, ENOTSOCK, ENOTCONN, ENOBUFS, EFAULT, EMSGSIZE, EAGAIN/EWOULDBLOCK
 *   EINTR, ENOMEM, EPIPE
 *
 * A few other more general winsock errors are included for good measure.
 * Note: text of error messages is very much a first cut and can probably be
 * imrpoved upon.  
 *
 */
static X25_ERRMAP const linux_errmap[] = {
    { EACCES,                   0,	
            "Permission denied" },
    { EADDRINUSE,               "java/net/BindException", 
            "X.25 address already in use" },
    { EAFNOSUPPORT,             0,  
            "Address family not supported by protocol family" },
    { EALREADY,                 0,  
            "Operation already in progress" },
    { ECONNREFUSED,             "java/net/ConnectException", 
            "X.25 Call Cleared" },
    { EFAULT,                   0, 
            "Bad address" },
    { EISCONN,                  0, 	
            "Socket is already connected" },
    { EMFILE,                   0,	
            "Too many open files" },
    { EMSGSIZE,                 0, 	
            "The message is larger than the maximum supported by the underlying transport" },
    { ENETUNREACH,              0, 	
            "X.25 line stopped or PVC line or channel number invalid" },
    { ENOBUFS,               0, 	
            "No buffer space available" },
    { ENOTSOCK,              0, 	
            "Socket operation on nonsocket" },
    { EOPNOTSUPP,            0, 	
            "Operation not supported" },
    { EPROTONOSUPPORT,       0, 	
            "Protocol not supported" },
    { EINVAL,                0, 
            "Invalid argument" },
    { ETIMEDOUT,                "java/net/ConnectException", 	
            "No repsonse to X.25 call within the T21 timeout period." },
    { ENOMEM,                   0, 	
            "Insufficient memory available" },
    { ENFILE,                   0,
            "Not enough memory to create new socket." },
    { ENODEV,                   0,
            "No FarSync X.25 cards installed." },
    { EBADF,                    0,
            "Socket operation on nonsocket descriptor." },
    { ENOSPC,                   0, 	
            "Not enough resources on FarSync card." },
    { EINTR,                    "java/io/InterruptedIOException", 	
            "Operation interrupted." },
    { EAGAIN,                   0, 	
            "No more free local ports." },
    { EIO,                      0, 	
            "An I/O error occured." },
    { ENOTCONN,                 0, 	
            "Socket is not connected." },
    { EPIPE,                    0, 	
            "Socket has been shutdown." },
    // !! END OF TABLE MARKER - DO NOT REMOVE !!
    { 0, 0, 0 },
};


int PLAT_close(int sd)
{
    return close(sd);
}


int PLAT_cudsetcmd(jboolean clr)
{
    return (clr) ? SIOCX25SCLEARUSERDATA : SIOCX25SCALLUSERDATA;
}


int PLAT_errno()
{
    return errno;
}


X25_ERRMAP const *PLAT_errmap()
{
    return linux_errmap;
}


int PLAT_ioctl(int sd, long cmd, void *args, int argLen)
{
    return ioctl(sd, cmd, args);
}


int PLAT_retryable()
{
    // return whether it may be possible to retry an operation afer an error
    // condition
    int err = PLAT_errno();
    return (err == ENOBUFS || err == ENOMEM) ? -1 : 0;
}

