/*
 *                 Common Public License Notice
 * 
 * The contents of this file are subject to the Common Public License
 * Version 1.0 (the "License"). You may not use this file except in
 * compliance with the License. A copy of the License should have been 
 * provided in the release which contained this file. If none was provided,
 * copies are available at http://www.opensource.org/
 *
 * Copyright (c) 2003 Ascert LLC.
 *
 * Contact: Ascert LLC (info@ascert.com)
 * Contributor(s):
 *      Rob Walker, Ascert LLC
 *
 */

#include <stdio.h>
#include <stdlib.h>
#include <sys/types.h>

#ifdef __linux__ 
#include <errno.h>
#include <string.h>
#include <sys/ioctl.h>
#include <sys/socket.h>
#include <linux/socket.h>
#include <linux/sockios.h>
#include <linux/x25.h>
#include <fsx25sockapi.h>
#endif

#ifdef __win32__
#include <windows.h>
#include <winsock2.h>
#include <ctype.h>
#include <malloc.h>
#include <ws2farsync.h>
#endif


#include "com_ascert_comms_driver_farsite_X25SocketImpl.h"

#include "x25utl.h"
#include "platform.h"

/************************************************************************
 * X25SocketImpl
 */

//TODO: use jboolean ExceptionCheck(JNIEnv *env); on "throw new" 
//      also ExceptionOccured ?? 
 
jfieldID psi_sdID; 

/* diagnostic indicators */
jboolean diag;

/* platform specific error map */
X25_ERRMAP *errmap;

/*
 * Called whenever X25SocketImpl is loaded. Caches field IDs for performance.
 *
 * Class:     com_ascert_comms_driver_farsite_X25SocketImpl
 * Method:    initImpl
 * Signature: ()V
 */
JNIEXPORT void JNICALL Java_com_ascert_comms_driver_farsite_X25SocketImpl_initImpl
  (JNIEnv *env, jclass cls)
{
    jfieldID tmpID; 
    tmpID = (*env)->GetStaticFieldID(env, cls, "diag", "Z");
    diag = (*env)->GetStaticBooleanField(env, cls, tmpID); 
    if (diag)
    {
        printf("[diag] field id = %d, value = %d\n", tmpID, diag);
    }

    psi_sdID = (*env)->GetFieldID(env, cls, "sd", "I");
    if (diag)
    {
        printf("[sd] field id = %d\n", psi_sdID);
    }

    /* set platform specific error map */
    errmap = PLAT_errmap();
}

/*
 * Create an X.25 socket and set the class "sd" field to contain it's
 * file descriptor.
 * 
 * Class:     com_ascert_comms_driver_farsite_X25SocketImpl
 * Method:    socketCreate
 * Signature: ()V
 */
JNIEXPORT void JNICALL Java_com_ascert_comms_driver_farsite_X25SocketImpl_fskCreate
  (JNIEnv *env, jobject this)
{
    int retval;
    int sd;
    int optval;
    
    sd = socket(AF_X25, SOCK_SEQPACKET, 0);
    
    if (sd == -1) 
    {
        if (diag)
        {
            printf("Error creating socket: %d\n", PLAT_errno());
        }
        UTL_throw(env, errmap, PLAT_errno(), "socket create", NULL);
        return;
    } 

    optval =  1;
    retval = setsockopt(sd, SOL_X25, X25_QBITINCL, &optval, sizeof(int));
    if (retval != 0)
    {
        if (diag)
        {
            printf("Error setting M/Q header option: sd=%d, err=%d\n", 
                    sd, PLAT_errno());
        }
        UTL_throw(env, errmap, PLAT_errno(), "setting Q bit header option",
                NULL);
        return;
    }

    optval =  1;
    retval = setsockopt(sd, SOL_X25, X25_MBITINCL, &optval, sizeof(int));
    if (retval != 0)
    {
        if (diag)
        {
            printf("Error setting M/Q header option: sd=%d, err=%d\n", 
                    sd, PLAT_errno());
        }
        UTL_throw(env, errmap, PLAT_errno(), "setting M bit header option",
                NULL);
        return;
    }
    
    
    if (diag)
    {
        printf("Socket created: sd=%d\n", sd);
    }
    (*env)->SetIntField(env, this, psi_sdID, sd);
}

/*
 * Bind the current socket to a local address.
 * 
 * Class:     com_ascert_comms_driver_farsite_X25SocketImpl
 * Method:    socketBind
 * Signature: (Ljava/lang/String;)V
 */
JNIEXPORT void JNICALL Java_com_ascert_comms_driver_farsite_X25SocketImpl_fskBind
  (JNIEnv *env, jobject this, jstring localAddr)
{
    int retval;
    int sd = (*env)->GetIntField(env, this, psi_sdID);
    const char *addr = (*env)->GetStringUTFChars(env, localAddr, 0);

    struct sockaddr_x25 bindAddr;
    bindAddr.sx25_family = AF_X25;
    strcpy(bindAddr.sx25_addr.x25_addr, addr);
    retval = bind(sd, &bindAddr, sizeof(struct sockaddr_x25));
    
    if (retval == -1) 
    {
        if (diag)
        {
            printf("Error binding socket: %d\n", PLAT_errno());
        }
        UTL_throw(env, errmap, PLAT_errno(), "socket bind", NULL);
        (*env)->ReleaseStringUTFChars(env, localAddr, addr);
        return;
    }
    
    if (diag)
    {
        printf("Socket bound: sd=%d, addr=%s\n", sd, addr);
    }
    (*env)->ReleaseStringUTFChars(env, localAddr, addr);
}

/*
 * Post a listen on this socket
 *
 * Class:     com_ascert_comms_driver_farsite_X25SocketImpl
 * Method:    socketListen
 * Signature: (I)V
 */
JNIEXPORT void JNICALL Java_com_ascert_comms_driver_farsite_X25SocketImpl_fskListen
  (JNIEnv *env, jobject this, jint backlog)
{
    int retval;
    int sd = (*env)->GetIntField(env, this, psi_sdID);
    retval = listen(sd, backlog);

    if (retval == -1) 
    {
        if (diag)
        {
            printf("Error listening on socket: sd=%d, err=%d\n", sd,
                    PLAT_errno());
        }
        UTL_throw(env, errmap, PLAT_errno(), "socket listen", NULL);
        return;
    }
    
    if (diag)
    {
        printf("Socket listen posted: sd=%d\n", sd);
    }
}

/*
 * Post an accept to wait for incoming connections on a server socket.
 *
 * Class:     com_ascert_comms_driver_farsite_X25SocketImpl
 * Method:    socketAccept
 * Signature: ()I
 */
JNIEXPORT jint JNICALL Java_com_ascert_comms_driver_farsite_X25SocketImpl_fskAccept
  (JNIEnv *env, jobject this)
{
    //TODO: timeout handling code
    int sd = (*env)->GetIntField(env, this, psi_sdID);
    int addrlen;
    int sdNew;
    struct sockaddr_x25 remoteAddr;
    
    addrlen = sizeof(struct sockaddr_x25);
    sdNew = accept(sd, &remoteAddr, &addrlen);

    if (sdNew == -1) 
    {
        if (diag)
        {
            printf("Error on accept: sd=%d, err=%d\n", sd, PLAT_errno());
        }
        UTL_throw(env, errmap, PLAT_errno(), "socket accept", NULL);
        return -1;
    }
    
    if (diag)
    {
        printf("Accepted new socket: sd=%d, sdNew=%d\n", sd, sdNew);
    }
    return sdNew;
}

/*
 * Establish a new client socket connection to a specified remote NUA
 *
 * Class:     com_ascert_comms_driver_farsite_X25SocketImpl
 * Method:    socketConnect
 * Signature: (Ljava/lang/String;I)V
 */
JNIEXPORT void JNICALL Java_com_ascert_comms_driver_farsite_X25SocketImpl_fskConnect
  (JNIEnv *env, jobject this, jstring remoteAddr, jint timeout)
{
    //TODO: timeout handling code
    int retval;
    int sd = (*env)->GetIntField(env, this, psi_sdID);
    const char *addr = (*env)->GetStringUTFChars(env, remoteAddr, 0);
    struct sockaddr_x25 toAddr;
    
    toAddr.sx25_family = AF_X25;
    strcpy(toAddr.sx25_addr.x25_addr, addr);
    retval = connect(sd, &toAddr, sizeof(struct sockaddr_x25));
    
    if (retval == -1) 
    {
        if (diag)
        {
            printf("Error connecting socket: sd=%d, err=%d\n", sd,
                    PLAT_errno());
        }
        UTL_throw(env, errmap, PLAT_errno(), "socket connect", NULL);
        (*env)->ReleaseStringUTFChars(env, remoteAddr, addr);
        return;
    } 

    if (diag)
    {
        printf("Socket connected: sd=%d, addr=%s\n", sd, addr);
    }
    (*env)->ReleaseStringUTFChars(env, remoteAddr, addr);
}

/*
 * Close an X.25 socket.
 *
 * Class:     com_ascert_comms_driver_farsite_X25SocketImpl
 * Method:    socketClose
 * Signature: ()V
 */
JNIEXPORT void JNICALL Java_com_ascert_comms_driver_farsite_X25SocketImpl_fskClose
  (JNIEnv *env, jobject this)
{
    int retval;
    int sd = (*env)->GetIntField(env, this, psi_sdID);

    if (sd != -1) 
    {
        (*env)->SetIntField(env, this, psi_sdID, -1);
        retval = PLAT_close(sd);
    }

    if (retval == -1) 
    {
        if (diag)
        {
            printf("Error closing socket: sd=%d, err=%d\n", sd, PLAT_errno());
        }
        UTL_throw(env, errmap, PLAT_errno(), "socket close", NULL);
        return;
    }

    if (diag)
    {
        printf("Socket close: sd=%d\n", sd);
    }
}
  
/*
 * Shutdown an X.25 socket
 * 
 * Class:     com_ascert_comms_driver_farsite_X25SocketImpl
 * Method:    socketShutdown
 * Signature: ()V
 */
JNIEXPORT void JNICALL Java_com_ascert_comms_driver_farsite_X25SocketImpl_fskShutdown
  (JNIEnv *env, jobject this)
{
    int retval;
    int sd = (*env)->GetIntField(env, this, psi_sdID);
    
    // X.25 socket takes any shutdown as a "both" shutdown
    retval = shutdown(sd, 2);

    if (retval == -1) 
    {
        if (diag)
        {
            printf("Error on socket shutdown: sd=%d, err=%d\n", sd, 
                    PLAT_errno());
        }
        UTL_throw(env, errmap, PLAT_errno(), "socket shutdown", NULL);
        return;
    }

    if (diag)
    {
        printf("Socket shutdown: sd=%d\n", sd);
    }
}

/*
 * Send a data from a buffer on our X.25 socket.
 *
 * Class:     com_ascert_comms_driver_farsite_X25SocketImpl
 * Method:    socketSend
 * Signature: ([BI)I
 */
JNIEXPORT jint JNICALL Java_com_ascert_comms_driver_farsite_X25SocketImpl_fskSend
  (JNIEnv *env, jobject this, jbyteArray bufArr, jint len)
{
    int retval;
    int sd = (*env)->GetIntField(env, this, psi_sdID);
    jbyte *buf = (*env)->GetByteArrayElements(env, bufArr, 0); 

    retval = send(sd, buf, len, 0);

    if (retval == -1) 
    {
        if (diag)
        {
            printf("Error on socket send: sd=%d, err=%d\n", sd, PLAT_errno());
        }
        UTL_throw(env, errmap, PLAT_errno(), "socket send", NULL);
        (*env)->ReleaseByteArrayElements(env, bufArr, buf, JNI_ABORT);
        return -1;            
    }

    if (diag)
    {
        printf("Socket send: sd=%d, buf-len=%d, sent-len=%d\n", sd, len,
                retval);
    }
    (*env)->ReleaseByteArrayElements(env, bufArr, buf, JNI_ABORT);            
    return retval;
}

/*
 * Receive data into a buffer on our X.25 socket.
 * 
 * Class:     com_ascert_comms_driver_farsite_X25SocketImpl
 * Method:    socketRecv
 * Signature: ([BI)I
 */
JNIEXPORT jint JNICALL Java_com_ascert_comms_driver_farsite_X25SocketImpl_fskRecv
  (JNIEnv *env, jobject this, jbyteArray bufArr, jint len)
{
    int retval;
    int sd = (*env)->GetIntField(env, this, psi_sdID);
    jbyte *buf = (*env)->GetByteArrayElements(env, bufArr, 0);
    
    retval = recv(sd, buf, len, 0);

    if (retval == -1) 
    {
        if (diag)
        {
            printf("Error on socket recv: sd=%d, err=%d\n", sd, PLAT_errno());
        }
        UTL_throw(env, errmap, PLAT_errno(), "socket recv", NULL);
        (*env)->ReleaseByteArrayElements(env, bufArr, buf, 0);            
        return -1;
    }

    if (diag)
    {
        printf("Socket recv: sd=%d, buf-len=%d, recv-len=%d\n", sd, len,
                retval);
    }
    (*env)->ReleaseByteArrayElements(env, bufArr, buf, 0);            
    return retval;
}

/*
 * Get any call user data (CUD) for a socket 
 *
 * Class:     com_ascert_comms_driver_farsite_X25SocketImpl
 * Method:    fskGetCallUserData
 * Signature: ()[B
 */
JNIEXPORT jbyteArray JNICALL Java_com_ascert_comms_driver_farsite_X25SocketImpl_fskGetCUD
  (JNIEnv *env, jobject this, jboolean clr)
{
    int sd = (*env)->GetIntField(env, this, psi_sdID);
    struct x25_calluserdata cud;
    jbyteArray cudArr;
    jbyte *buf;
    
    if (PLAT_ioctl(sd, SIOCX25GCALLUSERDATA, &cud, sizeof(cud)))
    {
        if (diag)
        {
            printf("Socket ioctl err (get CUD): sd=%d, err=%d\n", sd,
                    PLAT_errno());
        }
        UTL_throw(env, errmap, PLAT_errno(), "get call user data", NULL);
        return NULL;
    }

    cudArr = (*env)->NewByteArray(env, cud.cudlength);
    buf = (*env)->GetByteArrayElements(env, cudArr, 0);
    memcpy(buf, &cud.cuddata, cud.cudlength);
    (*env)->ReleaseByteArrayElements(env, cudArr, buf, 0);
    return cudArr;
}

/*
 * Set call user data (CUD) on a socket
 * 
 * Class:     com_ascert_comms_driver_farsite_X25SocketImpl
 * Method:    fskSetCallUserData
 * Signature: ([B)V
 */
JNIEXPORT void JNICALL Java_com_ascert_comms_driver_farsite_X25SocketImpl_fskSetCUD
  (JNIEnv *env, jobject this, jbyteArray cudArr, jboolean clr)
{
    int sd = (*env)->GetIntField(env, this, psi_sdID);
    struct x25_calluserdata cud;
    jbyte *buf = (*env)->GetByteArrayElements(env, cudArr, 0); 
    
    cud.cudlength = (*env)->GetArrayLength(env, cudArr);
    memcpy(&cud.cuddata, buf, cud.cudlength);
    (*env)->ReleaseByteArrayElements(env, cudArr, buf, JNI_ABORT);    

    if (PLAT_ioctl(sd, PLAT_cudsetcmd(clr), &cud, sizeof(cud)))
    {
        if (diag)
        {
            printf("Socket ioctl err (set CUD): sd=%d, err=%d\n", sd, 
                    PLAT_errno());
        }
        UTL_throw(env, errmap, PLAT_errno(), "set call user data", NULL);
        return;
    }
}

/*
 * Get any call clearing cause and diagnostic codes for a socket.
 *
 * Class:     com_ascert_comms_driver_farsite_X25SocketImpl
 * Method:    fskGetClearCodes
 * Signature: ()[B
 */
JNIEXPORT jbyteArray JNICALL Java_com_ascert_comms_driver_farsite_X25SocketImpl_fskGetClearCodes
  (JNIEnv *env, jobject this)
{
    int sd = (*env)->GetIntField(env, this, psi_sdID);
    struct x25_causediag clr;
    jbyteArray clrArr;
    jbyte *buf;

    if (PLAT_ioctl(sd, SIOCX25GCAUSEDIAG, &clr, sizeof(clr)))
    {
        if (diag)
        {
            printf("Socket ioctl err (get clr diags): sd=%d, err=%d\n", sd, 
                    PLAT_errno());
        }
        UTL_throw(env, errmap, PLAT_errno(), "get clear codes", NULL);
        return NULL;
    }

    clrArr = (*env)->NewByteArray(env, 2);
    buf = (*env)->GetByteArrayElements(env, clrArr, 0);
    buf[0] = clr.cause;
    buf[1] = clr.diagnostic;
    (*env)->ReleaseByteArrayElements(env, clrArr, buf, 0);
    return clrArr;
}

/*
 * Set call clearing cause and diagnostic codes on a socket.
 *
 * Class:     com_ascert_comms_driver_farsite_X25SocketImpl
 * Method:    fskSetClearCodes
 * Signature: ([B)V
 */
JNIEXPORT void JNICALL Java_com_ascert_comms_driver_farsite_X25SocketImpl_fskSetClearCodes
  (JNIEnv *env, jobject this, jbyteArray clrArr)
{
    int sd = (*env)->GetIntField(env, this, psi_sdID);
    struct x25_causediag clr;
    jbyte *buf = (*env)->GetByteArrayElements(env, clrArr, 0);
    
    clr.cause = buf[0];
    clr.diagnostic  = buf[1];
    (*env)->ReleaseByteArrayElements(env, clrArr, buf, JNI_ABORT);    

    if (PLAT_ioctl(sd, SIOCX25SCAUSEDIAG, &clr, sizeof(clr)))
    {
        if (diag)
        {
            printf("Socket ioctl err (set clr diags): sd=%d, err=%d\n", sd, 
                    PLAT_errno());
        }
        UTL_throw(env, errmap, PLAT_errno(), "set clear codes", NULL);
        return;
    }
}

/*
 * Get any facilities for a socket in 'raw' format i.e. as an encoded byte
 * buffer using the standard X.25 facilities octet codes and formats. 
 *
 * Class:     com_ascert_comms_driver_farsite_X25SocketImpl
 * Method:    fskGetRawFacilities
 * Signature: ()[B
 */
JNIEXPORT jbyteArray JNICALL Java_com_ascert_comms_driver_farsite_X25SocketImpl_fskGetRawFacilities
  (JNIEnv *env, jobject this)
{
    int sd = (*env)->GetIntField(env, this, psi_sdID);
    struct fsk_raw_facs facs;
    jbyteArray facArr;
    jbyte *buf;

    if (PLAT_ioctl(sd, SIOCX25GRAWFAC, &facs, sizeof(facs)))
    {
        if (diag)
        {
            printf("Socket ioctl err (get raw facilities): sd=%d, err=%d\n", 
                    sd, PLAT_errno());
        }
        UTL_throw(env, errmap, PLAT_errno(), "get raw facilities", NULL);
        return NULL;
    }
    
    facArr = (*env)->NewByteArray(env, facs.length);
    buf = (*env)->GetByteArrayElements(env, facArr, 0);
    memcpy(buf, &facs.data, facs.length);
    (*env)->ReleaseByteArrayElements(env, facArr, buf, 0);
    return facArr;
}

/*
 * Set facilities on a socket in 'raw' format i.e. from an encoded byte
 * buffer using the standard X.25 facilities octet codes and formats.
 * 
 * Class:     com_ascert_comms_driver_farsite_X25SocketImpl
 * Method:    fskSetRawFacilities
 * Signature: ([B)V
 */
JNIEXPORT void JNICALL Java_com_ascert_comms_driver_farsite_X25SocketImpl_fskSetRawFacilities
  (JNIEnv *env, jobject this, jbyteArray facArr)
{
    int sd = (*env)->GetIntField(env, this, psi_sdID);
    struct fsk_raw_facs facs;
    jbyte *buf = (*env)->GetByteArrayElements(env, facArr, 0);
    
    facs.length = (*env)->GetArrayLength(env, facArr);
    memcpy(&facs.data, buf, facs.length);
    (*env)->ReleaseByteArrayElements(env, facArr, buf, JNI_ABORT);    

    if (PLAT_ioctl(sd, SIOCX25SRAWFAC, &facs, sizeof(facs)))
    {
        if (diag)
        {
            printf("Socket ioctl err (set raw facilities): sd=%d, err=%d\n", 
                    sd, PLAT_errno());
        }
        UTL_throw(env, errmap, PLAT_errno(), "set raw facilities", NULL);
        return;
    }
}

/*
 * Get the local or remote address for a socket (note: state is checked by the 
 * calling Java class).
 * 
 * Class:     com_ascert_comms_driver_farsite_X25SocketImpl
 * Method:    fskGetAddr
 * Signature: (Z)Ljava/lang/String;
 */
JNIEXPORT jstring JNICALL Java_com_ascert_comms_driver_farsite_X25SocketImpl_fskGetAddr
  (JNIEnv *env, jobject this, jboolean local)
{
    int retval;
    int sd = (*env)->GetIntField(env, this, psi_sdID);
    struct sockaddr_x25 nua;
    int nuaLen;
    char tmp[17];
    
    nuaLen = sizeof(nua);
    if (local)
    {
        
        retval = getsockname(sd, &nua, &nuaLen);
    }
    else
    {
        retval = getpeername(sd, &nua, &nuaLen);
    }

    if (retval == -1)
    {
        if (diag)
        {
            printf("Socket err (get nua): sd=%d, err=%d\n", sd, 
                    PLAT_errno());
        }
        UTL_throw(env, errmap, PLAT_errno(), "get NUA", NULL);
        return NULL;
    }
    
    // make sure we have a null at the end
    memset(tmp, 0, 17);
    strncpy(tmp, &nua.sx25_addr.x25_addr, 16);
    return (*env)->NewStringUTF(env, tmp);
}


/*
 * Get any physical adapter and line assignment for the socket.
 *
 * Class:     com_ascert_comms_driver_farsite_X25SocketImpl
 * Method:    fskGetAdapter
 * Signature: ()[I
 */
JNIEXPORT jintArray JNICALL Java_com_ascert_comms_driver_farsite_X25SocketImpl_fskGetAdapter
  (JNIEnv *env, jobject this)
{
    int sd = (*env)->GetIntField(env, this, psi_sdID);
    struct fsk_port_info inf;
    jintArray adaptArr;
    jint *buf;

    if (PLAT_ioctl(sd, SIOCX25GADAPTER, &inf, sizeof(inf))) 
    {
        if (diag)
        {
            printf("Socket ioctl err (get adapter): sd=%d, err=%d\n", 
                    sd, PLAT_errno());
        }
        UTL_throw(env, errmap, PLAT_errno(), "get adapter", NULL);
        return NULL;
    }
    
    adaptArr = (*env)->NewIntArray(env, 2);
    buf = (*env)->GetIntArrayElements(env, adaptArr, 0);
    buf[0] = inf.adapter;
    buf[1] = inf.line;
    (*env)->ReleaseIntArrayElements(env, adaptArr, buf, 0);
    return adaptArr;
}

/*
 * Set the physical adapter and line assignment for the socket.
 *
 * Class:     com_ascert_comms_driver_farsite_X25SocketImpl
 * Method:    fskSetAdapter
 * Signature: ([I)V
 */
JNIEXPORT void JNICALL Java_com_ascert_comms_driver_farsite_X25SocketImpl_fskSetAdapter
  (JNIEnv *env, jobject this, jintArray adaptArr)
{
    int sd = (*env)->GetIntField(env, this, psi_sdID);
    struct fsk_port_info inf;
    jint *buf = (*env)->GetIntArrayElements(env, adaptArr, 0);
    
    inf.adapter = buf[0];
    inf.line  = buf[1];
    (*env)->ReleaseIntArrayElements(env, adaptArr, buf, JNI_ABORT);    

    if (PLAT_ioctl(sd, SIOCX25SADAPTER, &inf, sizeof(inf))) 
    {
        if (diag)
        {
            printf("Socket ioctl err (set adapter): sd=%d, err=%d\n", 
                    sd, PLAT_errno());
        }
        UTL_throw(env, errmap, PLAT_errno(), "set adapter", NULL);
        return;
    }
}

/*
 * Gets the receive buffer size by reading SO_RCVBUF.
 * 
 * Class:     com_ascert_comms_driver_farsite_X25SocketImpl
 * Method:    fskGetRcvBuf
 * Signature: ()I
 */
JNIEXPORT jint JNICALL Java_com_ascert_comms_driver_farsite_X25SocketImpl_fskGetRcvBuf
  (JNIEnv *env, jobject this)
{
    int sd = (*env)->GetIntField(env, this, psi_sdID);
    int retval;
    int size;
    int len = sizeof(size);
    
// temporary for now, as FarSync may be making these calls compatible    
#ifdef __linux__ 
    retval = getsockopt(sd, SOL_X25, SO_RCVBUF, &size, &len);
#endif
#ifdef __win32__
    retval = getsockopt(sd, SOL_SOCKET, SO_RCVBUF, &size, &len);
#endif
    if (retval == -1)
    {
        if (diag)
        {
            printf("Socket err (get recv buf): sd=%d, err=%d\n", sd, 
                    PLAT_errno());
        }
        UTL_throw(env, errmap, PLAT_errno(), "getting receive buffer size", NULL);
        return -1;
    }

    if (diag)
    {
        printf("Socket get recv bufsize: sd=%d, SO_RCVBUF=%d\n", sd, size);
    }
    return size;
}

/*
 * Sets the receive buffer size by setting SO_RCVBUF.
 *
 * Class:     com_ascert_comms_driver_farsite_X25SocketImpl
 * Method:    fskSetRcvBuf
 * Signature: (I)V
 */
JNIEXPORT void JNICALL Java_com_ascert_comms_driver_farsite_X25SocketImpl_fskSetRcvBuf
  (JNIEnv *env, jobject this, jint pSize)
{
    int sd = (*env)->GetIntField(env, this, psi_sdID);
    int retval;
    int size = pSize;

// temporary for now, as FarSync may make these calls compatible    
#ifdef __linux__ 
    retval = setsockopt(sd, SOL_X25, SO_RCVBUF, &size, sizeof(size));
#endif
#ifdef __win32__
    retval = setsockopt(sd, SOL_SOCKET, SO_RCVBUF, &size, sizeof(size));
#endif

    if (retval == -1)
    {
        if (diag)
        {
            printf("Socket err (set recv buf): sd=%d, err=%d\n", sd, 
                    PLAT_errno());
        }
        UTL_throw(env, errmap, PLAT_errno(), "setting receive buffer size", NULL);
        return;
    }

    if (diag)
    {
        printf("Socket set recv bufsize: sd=%d, SO_RCVBUF=%d\n", sd, pSize);
    }
}


