/*
 *                 Common Public License Notice
 * 
 * The contents of this file are subject to the Common Public License
 * Version 1.0 (the "License"). You may not use this file except in
 * compliance with the License. A copy of the License should have been 
 * provided in the release which contained this file. If none was provided,
 * copies are available at http://www.opensource.org/
 *
 * Copyright (c) 2003 Ascert LLC.
 *
 * Contact: Ascert LLC (info@ascert.com)
 * Contributor(s):
 *      Rob Walker, Ascert LLC
 *
 */

#include <winsock2.h>

#include "jni.h"
#include <ws2farsync.h>

#include "x25utl.h"
#include "platform.h"

int (PASCAL FAR* WSAStartup_fn)(WORD, LPWSADATA);
int (PASCAL FAR* WSACleanup_fn)();
int (PASCAL FAR *ioctlsocket_fn)();

/*
 * Table used to map known error conditions, to appropriate error messages 
 * and exception types. The following are the main specific errors that the 
 * farsite api will return:
 * 
 *  Bind-type operations (includes connecting on unbound sockets):
 *   WSAEADDRINUSE, WSAENETDOWN, WSAEADDRNOTAVAIL, WSAENOBUFS, WSAEFAULT
 *  
 *  Listen-type operations:
 *   WSAENETUNREACH, WSAENETDOWN, WSAEISCONN, WSAENOBUFS
 *  
 *  Connect-type operations:
 *   NO_ADDRESS, WSAENETUNREACH, WSAECONNREFUSED, WSAEADDRINUSE, WSAECONNREFUSED
 *   WSAETIMEDOUT, WSAENETRESET, WSAEHOSTUNREACH, WSAENETDOWN, WSAEALREADY
 *   WSAEISCONN, WSAENOBUFS
 * 
 *  Data transfer operations:
 *   WSAECONNRESET, WSAENETDOWN, WSAENETUNREACH, WSAENETRESET, WSAECONNABORTED
 *   WSAEDISCON, WSAENETUNREACH, WSAENOBUFS
 *
 * A few other more general winsock errors are included for good measure.
 * Note: text of error messages is very much a first cut and can probably be
 * imrpoved upon.  
 *
 */
static X25_ERRMAP const win_errmap[] = {
    { WSANO_ADDRESS,            0, 
            "X.25 driver address resource not available" },
    { WSAEACCES,                0,	
            "Permission denied" },
    { WSAEADDRINUSE,            "java/net/BindException", 
            "X.25 address already in use" },
    { WSAEADDRNOTAVAIL,         "java/net/BindException", 
            "Cannot assign requested X.25 address" },
    { WSAEAFNOSUPPORT,          0, 
            "Address family not supported by protocol family" },
    { WSAEALREADY,              0, 
            "Operation already in progress" },
    { WSAECONNABORTED,          0, 
            "Software caused connection abort" },
    { WSAECONNREFUSED,          "java/net/ConnectException", 
            "X.25 Call Cleared" },
    { WSAECONNRESET,            0, 
            "X.25 Connection reset" },
    { WSAEFAULT,                0,
            "Bad address" },
    { WSAEHOSTUNREACH,          "java/net/NoRouteToHostException",	
            "No route to host" },
    { WSAEISCONN,               0,	
            "Socket is already connected" },
    { WSAEMFILE,                0,	
            "Too many open files" },
    { WSAEMSGSIZE,              0,	
            "The message is larger than the maximum supported by the underlying transport" },
    { WSAENETDOWN,              0,	
            "X.25 network link down or restarted." },
    { WSAENETRESET,             0,	
            "X.25 protocol error" },
    { WSAENETUNREACH,           0,	
            "X.25 line stopped or PVC line or channel number invalid" },
    { WSAENOBUFS,               0,	
            "No buffer space available" },
    { WSAENOTSOCK,              0,	
            "Socket operation on nonsocket" },
    { WSAEOPNOTSUPP,            0,	
            "Operation not supported" },
    { WSAEPFNOSUPPORT,          0,	
            "Protocol family not supported" },
    { WSAEPROCLIM,              0,	
            "Too many processes" },
    { WSAEPROTONOSUPPORT,       0,	
            "Protocol not supported" },
    { WSAEINVAL,                0,
            "Invalid argument" },
    { WSAEPROTOTYPE,            0,	
            "Protocol wrong type for socket" },
    { WSAESHUTDOWN,		        0,	
            "Cannot send after socket shutdown" },
    { WSAESOCKTNOSUPPORT,       0,	
            "Socket type not supported" },
    { WSAETIMEDOUT,             "java/net/ConnectException",	
            "No repsonse to X.25 call within the T21 timeout period." },
    { WSA_NOT_ENOUGH_MEMORY,    0,	
            "Insufficient memory available" },
    { WSANOTINITIALISED,        0,	
            "Successful WSAStartup not yet performed" },
    { WSASYSNOTREADY,           0,	
            "Network subsystem is unavailable" },
    { WSAVERNOTSUPPORTED,       0,	
            "Winsock.dll version out of range" },
    { WSAEDISCON,               0,	
            "Normal X.25 disconnection." },
    // !! END OF TABLE MARKER - DO NOT REMOVE !!
    { 0, 0, 0 },
};

/*
 * Ensure our DLL is initialized for WinSock API support
 */
BOOL WINAPI
DllMain(HINSTANCE hinst, DWORD reason, LPVOID reserved)
{
    WSADATA wsadata;
    HANDLE hWinsock;
    WORD wReq;

    switch (reason) 
    {
        case DLL_PROCESS_ATTACH:
            // Try to load winsock2. 
            hWinsock = LoadLibrary("ws2_32.dll");
            if (hWinsock == NULL) 
            {
                return FALSE;
            }
    
            // Get addresses of the routines we require
            WSAStartup_fn = (int (PASCAL FAR *)(WORD, LPWSADATA))
                    GetProcAddress(hWinsock, "WSAStartup");
            WSACleanup_fn = (int (PASCAL FAR *)())GetProcAddress(hWinsock,
                    "WSACleanup");
            ioctlsocket_fn = (int (PASCAL FAR *)())GetProcAddress(hWinsock,
                    "ioctlsocket");
    
            // Initialize winsock
            wReq = MAKEWORD(2,2);
            if ((*WSAStartup_fn)(wReq, &wsadata) != 0) 
            {
                return FALSE;
            }
            //printf("WSA_startup\n");
            break;
    
        case DLL_PROCESS_DETACH:
            if (WSACleanup_fn) 
            {
                (*WSACleanup_fn)();
                //printf("WSA_cleanup\n");
            }
            break;
    
        default:
            break;
    }
    return TRUE;
}


int PLAT_close(int sd)
{
    return closesocket(sd);
}


int PLAT_cudsetcmd(jboolean clr)
{
    // windows variant seems to use same ioctl code
    return SIOCX25SCALLUSERDATA;
}


int PLAT_errno()
{
    return WSAGetLastError();
}


X25_ERRMAP const *PLAT_errmap()
{
    return win_errmap;
}


int PLAT_ioctl(int sd, long cmd, void *args, int argLen)
{
    long retLen;
    return WSAIoctl(sd, cmd, args, argLen, args, argLen, &retLen, NULL, NULL); 
}


int PLAT_retryable()
{
    // return whether it may be possible to retry an operation afer an error
    // condition
    return (PLAT_errno() == WSAENOBUFS) ? -1 : 0;
}

